/*
 * tnef.c -- extract files from microsoft TNEF format
 *
 * Copyright (C)1999, 2000 Mark Simpson <damned@world.std.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can either send email to this
 * program's maintainer or write to: The Free Software Foundation,
 * Inc.; 59 Temple Place, Suite 330; Boston, MA 02111-1307, USA.
 *
 * Commentary:
 * 	main function and command line parsing.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <memory.h>
#include <stdio.h>
#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
#include "getopt_long.h"
#endif /* HAVE_GETOPT_LONG */

#include "tnef.h"

/* COPYRIGHTS & NO_WARRANTY -- defined to make code below a little nicer to
   read */
static const char* COPYRIGHTS = \
"Copyright (C) 1999 by Mark Simpson\n"
"Copyright (C) 1997 by Thomas Boll (original code)";
static const char* NO_WARRANTY = \
"%s comes with ABSOLUTELY NO WARRANTY.\n"
"You may redistribute copies of %s under the terms of the GNU General\n"
"Public License.  For more information about these matters, see the file\n"
"named COPYING.";
static const char* USAGE = \
"-f FILE, --file=FILE\tuse FILE as input ('-' == stdin)\n"
"-d DIR, --directory=DIR\tput unpacked files in DIR\n"
"-l, --list          \tlist files, do not extract\n"
"-n, --dry_run       \tparse the file only\n"
"-v, --verbose       \tproduce verbose output\n"
"-h, --help          \tshow this message\n"
"--version           \tdisplay version and copyright\n"
"\nIf FILE is not specified standard input is used\n"
"\nReport bugs to <damned@world.std.com>\n";

/* Program Options: */
static int verbose = 0;         /* verbose output */
static int dry_run = 0;         /* don't actually do antying */
static char* out_dir = 0;       /* where to put output files */
static char* in_file = 0;       /* input file */



static void
usage (char* prog)
{
    fprintf (stderr, "%s: [options] [FILE]\n", prog);
    fprintf (stderr, USAGE);
}


static void
parse_cmdline (int argc, char **argv)
{
    int i = 0;
    int option_index = 0;
    static struct option long_options[] = 
    { 
        {"file", required_argument, 0, 'f' },
        {"directory", required_argument, 0, 'd' },
        {"help", no_argument, 0, 'h'},
        {"verbose", no_argument, 0, 'v'},
        {"dry_run", no_argument, 0, 'n'},
        {"list", no_argument, 0, 'l'}, /* for now same as -n */
        {"version", no_argument, 0, 0},
        { 0, 0, 0, 0 }
    };

    while ((i = getopt_long (argc, argv, "f:d:vhnl", 
                             long_options, &option_index)) != -1)
    {
        switch (i) 
        {
        case 0:               /* long options with no val field */
            if (strcmp (long_options[option_index].name, 
                             "version") == 0)
            {
                fprintf (stderr, "%s %s\n", PACKAGE, VERSION);
                fprintf (stderr, COPYRIGHTS);
                fprintf (stderr, "\n");
                fprintf (stderr, NO_WARRANTY, PACKAGE, PACKAGE);
                fprintf (stderr, "\n");
                exit (0);
            }
            else
            {
                abort ();       /* impossible! */
            }
            break;

        case 'v': 
            verbose = 1;
            break;

        case 'f':
            if (strcmp (optarg, "-") == 0)
                in_file = NULL;
            else in_file = optarg;
            break;
            
        case 'd':
            out_dir = optarg;
            break;

        case 'h':
            usage(argv[0]);
            exit (0);
            break;

        case 'l':               /* for now -l/--list same as -n */
        case 'n':               /* tell system to not do anything just go
                                   through the motions */
            dry_run = 1;
            break;

        case '?':
            exit (1);
            break;

        default:
            abort();            /* this is a problem */
        }
    }
    if (optind < argc)
    {
        in_file = argv[optind++];
        if (optind < argc)
        {
            fprintf (stderr, "%s: Extra parameters -- '", argv[0]);
            while (optind < argc)
                fprintf (stderr, "%s ", argv[optind++]);
            fprintf (stderr, "'\n");
        }
    }
}


int
main (int argc, char *argv[]) 
{
    FILE *fp = NULL;
    
    parse_cmdline (argc, argv);

    /* If user didn't specify directory use current directory */
    if (!out_dir)
    {
        out_dir = "./";
    }
    /* open the file */
    if (in_file)
    {
        fp = fopen (in_file, "rb");
        if (fp == NULL) 
        {
            perror (in_file);
            exit (1);
        }
    }
    else
    {
        fp = stdin;
    }
    
    return parse_file (fp, out_dir, verbose, dry_run);
}
